@@ -1,17 +1,3 @@
-##' invert the description of a multilayer to simulate the opposite direction of incidence
-##'
-##' inverts list of epsilon and thickness of layers
-##' @title invert_stack
-##' @param p list
-##' @return list
-##' @export
-##' @family helping_functions
-##' @author Baptiste Auguie
-invert_stack <- function(p){
-  p[["epsilon"]] <- rev(p[["epsilon"]])
-  p[["thickness"]] <- rev(p[["thickness"]])
-  p
-}
 
 ##' Multilayer Fresnel coefficients
 ##'
@@ -47,7 +33,8 @@ multilayer <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
   epsilon = do.call(cbind, epsilon)
   ## case pure scalars
   if(nrow(epsilon) == 1L)
-    epsilon <- matrix(epsilon, nrow=length(k0), ncol=length(thickness), byrow=TRUE)
+    epsilon <- matrix(epsilon, nrow=length(k0), 
+                      ncol=length(thickness), byrow=TRUE)
   
   ## define constants
   Nlambda <- length(k0)
@@ -302,7 +289,6 @@ multilayer <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
 ##' @param q [vector] normalised incident in-plane wavevector
 ##' @param epsilon list of N+2 dielectric functions, each of length 1 or length(wavelength)
 ##' @param thickness vector of N+2 layer thicknesses, first and last are dummy
-##' @param polarisation [character] switch between p- and s- polarisation
 ##' @param ... unused
 ##' @return fresnel coefficients and field profiles
 ##' @author baptiste Auguie
@@ -312,14 +298,16 @@ multilayer <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
 multilayercpp <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
                        angle = asin(q), q = sin(angle),
                        epsilon = list(incident=1.5^2, 1.33),
-                       thickness = c(0, 0),
-                       polarisation = c('p', 's'), ...){
+                       thickness = c(0, 0), ...){
 
   
   kx <- outer(k0*sqrt(epsilon[[1]]), q) # kx = q*k0
   epsilon = do.call(cbind, epsilon)
-  polarisation = if(polarisation == "p") 0L else 1L
-  
+  ## case pure scalars
+  if(nrow(epsilon) == 1L)
+    epsilon <- matrix(epsilon, nrow=length(k0), 
+                      ncol=length(thickness), byrow=TRUE)
+
   Nlayer <- length(thickness)
   Nlambda <- length(k0)
   Nq <- length(q)
@@ -334,42 +322,54 @@ multilayercpp <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
             ncol(kx) == length(q))
 
   ## call the C++ function
-  res <- planar$multilayer(as.vector(k0), as.matrix(kx), as.matrix(epsilon),
-                           as.vector(thickness), as.integer(polarisation))
+  res <- cpp_multilayer(as.vector(k0), as.matrix(kx), as.matrix(epsilon),
+                           as.vector(thickness), 
+                        0.0, # z=0 irrelevant don't calculate fields
+                        0.0, # psi=0 irrelevant don't calculate fields
+                        FALSE) # don't need intensities
   
-  transmission <- drop(res$transmission)
-  reflection <- drop(res$reflection)
+  ts <- drop(res$ts)
+  rs <- drop(res$rs)
+  tp <- drop(res$tp)
+  rp <- drop(res$rp)
   
   ## T is nt*cos(Ot)*|Et|^2 / ni*cos(Oi)*|Ei|^2
   ## for s-pol, |Et|^2 / |Ei|^2 = |ts|^2, hence T = nt/ni * cos(Ot)/cos(Oi) * |ts|^2
   ## for p-pol, |Et|^2 / |Ei|^2 = (ni/nt)^2 * |tp|^2, hence T = ni/nt * cos(Ot)/cos(Oi) * |tp|^2
   
-  
   # ratio of refractive indices
-  index.ratio <- matrix(Re(sqrt(epsilon[,1])/sqrt(epsilon[,Nlayer])), nrow=Nlambda, ncol=Nq)
+  index.ratio <- matrix(Re(sqrt(epsilon[,1])/sqrt(epsilon[,Nlayer])), 
+                        nrow=Nlambda, ncol=Nq)
   # ratio of cosines
   qq <- matrix(q, nrow=Nlambda, ncol=Nq, byrow=TRUE)
   m <- Re(sqrt(1 - (index.ratio * qq)^2 + 0i)/sqrt(1 - qq^2 + 0i))
   
-  if(polarisation == 0L){
-    rho <- index.ratio
-    reflection <- -reflection # sign convention was for H
-  } else {
-    rho <- 1 / index.ratio
-  }
+  dim(ts) <- dim(tp) <- dim(m) # case 1-dims were dropped
   
-  dim(transmission) <- dim(m) # case 1-dims were dropped
-  R <- Mod(reflection)^2
-  T <- rho * m * Mod(transmission)^2
-  T <- drop(T)
-  transmission <- drop(transmission)
+  # p-pol
+    rhop <- index.ratio
+    rp <- -rp # sign convention was for H
+  # s-pol
+    rhos <- 1 / index.ratio
   
-  A <- 1 - R - T
+  
+  Rp <- Mod(rp)^2
+  Rs <- Mod(rs)^2
+  Ts <- drop(rhos * m * Mod(ts)^2)
+  Tp <- drop(rhop * m * Mod(tp)^2)
+  
+  ts <- drop(ts)
+  tp <- drop(tp)
+  
+  As <- 1 - Rs - Ts
+  Ap <- 1 - Rp - Tp
   
   list(wavelength=wavelength, k0 = k0, 
        angle=angle, q=q, 
-       reflection=reflection, transmission=transmission,
-       R=R, T=T, A=A)
+       rs=rs, ts=ts,
+       rp=rp, tp=tp,
+       Rs=Rs, Ts=Ts, As=As,
+       Rp=Rp, Tp=Tp, Ap=Ap)
 }
 
 ##' Multilayer Fresnel coefficients
@@ -412,12 +412,12 @@ multilayerfull <- function(wavelength = 2*pi/k0, k0 = 2*pi/wavelength,
             ncol(kx) == length(q))
   
   ## call the C++ function
-  res <- planar$multilayerfull(as.vector(k0), 
+  res <- cpp_multilayer(as.vector(k0), 
                                as.matrix(kx), 
                                as.matrix(epsilon),
                                as.vector(thickness), 
                                as.vector(z),
-                               as.double(psi))
+                               as.double(psi), TRUE)
   
   ts <- drop(res$ts)
   rs <- drop(res$rs)  
